@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -63,6 +63,32 @@ enum {
 };
 
 
+typedef void (*IterateFunc) (guchar *src, gpointer extra_data);
+
+static void
+_gdk_pixbuf_iterate (GdkPixbuf   *src,
+		     gpointer     extra_data,
+		     IterateFunc  iterate_func)
+{
+        guint   h          = gdk_pixbuf_get_height (src);
+	guchar *pixels     = gdk_pixbuf_get_pixels (src);
+	int     n_channels = gdk_pixbuf_get_n_channels (src);
+	int     rowstride  = gdk_pixbuf_get_rowstride (src);
+
+	while (h--) {
+		guint   w = gdk_pixbuf_get_width (src);
+                guchar *p = pixels;
+                 
+		while (w--) {
+			(*iterate_func) (p, extra_data); 
+			p += n_channels;
+		}
+  
+		pixels += rowstride;
+	}
+}
+
+
 
 /* -- desaturate -- */
 
@@ -1148,3 +1174,188 @@ _gdk_pixbuf_adjust_levels (GdkPixbuf *src,
 				  adjust_levels_release,
 				  data);	
 }
+
+
+
+
+
+typedef struct {
+	int      alpha;
+	guchar   lut[256][3];
+	guchar   min[3];
+	guchar   max[3];
+	gboolean has_alpha;
+} StretchContrastData;
+
+
+static void
+stretch__find_min_max (guchar   *src,
+		       gpointer  extra_data)
+{
+	StretchContrastData *data = extra_data;
+	int b;
+
+	for (b = 0; b < data->alpha; b++) 
+		if (! data->has_alpha || src[data->alpha]) {
+			if (src[b] < data->min[b])
+				data->min[b] = src[b];
+			if (src[b] > data->max[b])
+				data->max[b] = src[b];
+		}
+}
+
+
+static void
+stretch_contrast_init (GthPixbufOp *pixop)
+{
+	StretchContrastData *data = pixop->data;
+	int b;
+
+	data->has_alpha = gdk_pixbuf_get_has_alpha (pixop->src);
+	data->alpha = gdk_pixbuf_get_n_channels (pixop->src);
+
+	data->min[0] = data->min[1] = data->min[2] = 255;
+	data->max[0] = data->max[1] = data->max[2] = 0;
+	_gdk_pixbuf_iterate (pixop->src, data, stretch__find_min_max);
+
+	/* Calculate LUTs with stretched contrast */
+	for (b = 0; b < data->alpha; b++) {
+		int range = data->max[b] - data->min[b];
+
+		if (range != 0) {
+			int x;
+			for (x = data->min[b]; x <= data->max[b]; x++)
+				data->lut[x][b] = 255 * (x - data->min[b]) / range;
+		} else
+			data->lut[data->min[b]][b] = data->min[b];
+	}
+}
+
+
+static void
+stretch_contrast_step (GthPixbufOp *pixop)
+{
+	StretchContrastData *data = pixop->data;
+	int b;
+
+	for (b = 0; b < data->alpha; b++)
+		pixop->dest_pixel[b] = data->lut[pixop->src_pixel[b]][b];
+
+	if (data->has_alpha)
+		pixop->dest_pixel[data->alpha] = pixop->src_pixel[data->alpha];
+}
+
+
+static void
+stretch_contrast_release (GthPixbufOp *pixop)
+{
+	g_free (pixop->data);
+}
+
+
+GthPixbufOp*
+_gdk_pixbuf_stretch_contrast (GdkPixbuf *src,
+			      GdkPixbuf *dest)
+{
+	StretchContrastData *data;
+
+	data = g_new (StretchContrastData, 1);
+
+	return gth_pixbuf_op_new (src, dest,
+				  stretch_contrast_init,
+				  stretch_contrast_step,
+				  stretch_contrast_release,
+				  data);	
+}
+
+
+
+
+
+typedef struct {
+	int      alpha;
+	guchar   lut[256];
+	guchar   min;
+	guchar   max;
+	gboolean has_alpha;
+} NormalizeContrastData;
+
+
+static void
+normalize__find_min_max (guchar   *src,
+			 gpointer  extra_data)
+{
+	NormalizeContrastData *data = extra_data;
+	int b;
+
+	for (b = 0; b < data->alpha; b++) 
+		if (! data->has_alpha || src[data->alpha]) {
+			if (src[b] < data->min)
+				data->min = src[b];
+			if (src[b] > data->max)
+				data->max = src[b];
+		}
+}
+
+
+static void
+normalize_contrast_init (GthPixbufOp *pixop)
+{
+	NormalizeContrastData *data = pixop->data;
+	int range;
+
+	data->has_alpha = gdk_pixbuf_get_has_alpha (pixop->src);
+	data->alpha = gdk_pixbuf_get_n_channels (pixop->src);
+
+	data->min = 255;
+	data->max = 0;
+	_gdk_pixbuf_iterate (pixop->src, data, normalize__find_min_max);
+
+	/* Calculate LUT */
+
+	range = data->max - data->min;
+
+	if (range != 0) {
+		int x;
+		for (x = data->min; x <= data->max; x++)
+			data->lut[x] = 255 * (x - data->min) / range;
+	} else
+		data->lut[data->min] = data->min;
+}
+
+
+static void
+normalize_contrast_step (GthPixbufOp *pixop)
+{
+	NormalizeContrastData *data = pixop->data;
+	int b;
+
+	for (b = 0; b < data->alpha; b++)
+		pixop->dest_pixel[b] = data->lut[pixop->src_pixel[b]];
+                                                                               
+	if (data->has_alpha)
+		pixop->dest_pixel[data->alpha] = pixop->src_pixel[data->alpha];
+}
+
+
+static void
+normalize_contrast_release (GthPixbufOp *pixop)
+{
+	g_free (pixop->data);
+}
+
+
+GthPixbufOp*
+_gdk_pixbuf_normalize_contrast (GdkPixbuf *src,
+				GdkPixbuf *dest)
+{
+	NormalizeContrastData *data;
+
+	data = g_new (NormalizeContrastData, 1);
+
+	return gth_pixbuf_op_new (src, dest,
+				  normalize_contrast_init,
+				  normalize_contrast_step,
+				  normalize_contrast_release,
+				  data);	
+}
