@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
@@ -146,16 +147,46 @@ comment_data_dup (CommentData *data)
 
 
 char *
-comments_get_comment_filename (const char *source) 
+comments_get_comment_filename (const char *source,
+			       gboolean    resolve_symlinks,
+			       gboolean    unescape) 
 {
-	char *path;
-	char *directory;
-	const char *filename;
+	char        *source_real = NULL;
+	char        *directory;
+	const char  *filename;
+	char        *path;
 
-	if (!source) return NULL;
+	if (source == NULL) 
+		return NULL;
 
-	directory = remove_level_from_path (source);
-	filename = file_name_from_path (source);
+	source_real = g_strdup (source);
+
+	if (resolve_symlinks) {
+		char           *parent;
+		char           *parent_escaped;
+		char           *parent_resolved;
+		GnomeVFSResult  result;
+
+		parent = remove_level_from_path (source);
+		parent_escaped = gnome_vfs_escape_path_string (parent);
+		g_free (parent);
+
+		result = resolve_all_symlinks (parent_escaped, 
+					       &parent_resolved);
+		g_free (parent_escaped);
+
+		if (result == GNOME_VFS_OK) { 
+			g_free (source_real);
+			source_real = g_strconcat (parent_resolved, 
+						   "/",
+						   file_name_from_path (source),
+						   NULL);
+			g_free (parent_resolved);
+		} 
+	}
+
+	directory = remove_level_from_path (source_real);
+	filename = file_name_from_path (source_real);
 
 	path = g_strconcat (g_get_home_dir(), 
 			    "/", 
@@ -167,29 +198,61 @@ comments_get_comment_filename (const char *source)
 			    NULL);
 
 	g_free (directory);
+	g_free (source_real);
+
+	if (unescape) {
+		char *unesc_path = gnome_vfs_unescape_string (path, NULL);
+		g_free (path);
+		path = unesc_path;
+	}
 
 	return path;
 }
 
 
 char *
-comments_get_comment_dir (const char *directory) 
+comments_get_comment_dir (const char *directory,
+			  gboolean    resolve_symlinks,
+			  gboolean    unescape) 
 {
-	char *path;
-	char  *separator;
+	char        *directory_resolved = NULL;
+	const char  *directory_real = directory;
+	const char  *separator;
+	char        *path;
 
-	if (directory == NULL)
+	if (resolve_symlinks && (directory != NULL)) {
+		char           *directory_escaped;
+		GnomeVFSResult  result;
+
+		directory_escaped = gnome_vfs_escape_path_string (directory);
+		result = resolve_all_symlinks (directory_escaped, 
+					       &directory_resolved);
+		g_free (directory_escaped);
+
+		if (result == GNOME_VFS_OK) 
+			directory_real = directory_resolved;
+	}
+
+	if (directory_real == NULL)
 		separator = NULL;
 	else
-		separator = (directory[0] == '/') ? "" : "/";
+		separator = (directory_real[0] == '/') ? "" : "/";
 
 	path = g_strconcat (g_get_home_dir(), 
 			    "/", 
 			    RC_COMMENTS_DIR, 
 			    separator,
-			    directory,
+			    directory_real,
 			    NULL);
 
+	g_free (directory_resolved);
+
+	if (unescape) {
+		char *unesc_path = gnome_vfs_unescape_string (path, NULL);
+		g_free (path);
+		path = unesc_path;
+	}
+
 	return path;
 }
 
@@ -201,8 +264,16 @@ comment_copy (const char *src,
 	char *comment_src;
 	char *comment_dest;
 
-	comment_src = comments_get_comment_filename (src);
-	comment_dest = comments_get_comment_filename (dest);
+	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
+	if (! path_is_file (comment_src)) {
+		g_free (comment_src);
+		comment_src = comments_get_comment_filename (src, FALSE, TRUE);
+		if (! path_is_file (comment_src)) {
+			g_free (comment_src);
+			return;
+		}
+	}
+	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
 
 	if (path_is_file (comment_dest)) 
 		unlink (comment_dest);
@@ -220,8 +291,17 @@ comment_move (const char *src,
 	char *comment_src;
 	char *comment_dest;
 
-	comment_src = comments_get_comment_filename (src);
-	comment_dest = comments_get_comment_filename (dest);
+	comment_src = comments_get_comment_filename (src, TRUE, TRUE);
+	if (! path_is_file (comment_src)) {
+		g_free (comment_src);
+		comment_src = comments_get_comment_filename (src, FALSE, TRUE);
+		if (! path_is_file (comment_src)) {
+			g_free (comment_src);
+			return;
+		}
+	}
+
+	comment_dest = comments_get_comment_filename (dest, TRUE, TRUE);
 
 	if (path_is_file (comment_dest)) 
 		unlink (comment_dest);
@@ -237,7 +317,11 @@ comment_delete (const char *filename)
 {
 	char *comment_name;
 
-	comment_name = comments_get_comment_filename (filename);
+	comment_name = comments_get_comment_filename (filename, FALSE, TRUE);
+	unlink (comment_name);
+	g_free (comment_name);
+
+	comment_name = comments_get_comment_filename (filename, TRUE, TRUE);
 	unlink (comment_name);
 	g_free (comment_name);
 }
@@ -249,8 +333,8 @@ comment_delete (const char *filename)
  */
 void
 comments_remove_old_comments (const char *dir,
-			      gboolean recursive,
-			      gboolean clear_all)
+			      gboolean    recursive,
+			      gboolean    clear_all)
 {
 	visit_rc_directory (RC_COMMENTS_DIR,
 			    COMMENT_EXT,
@@ -299,7 +383,7 @@ remove_comments_done (const GList *dir_list,
 	}
 
 	for (scan = dir_list; scan; scan = scan->next) {
-		gchar *dir = scan->data;
+		char *dir = scan->data;
 		rmdir (dir);
 	}
 }
@@ -325,6 +409,7 @@ comments_remove_old_comments_async (const char *dir,
 					       GTK_DIALOG_MODAL,
 					       GTK_MESSAGE_INFO,
 					       message,
+					       NULL,
 					       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					       NULL);
 	g_signal_connect_swapped (G_OBJECT (crd->dialog),
@@ -344,15 +429,16 @@ comments_remove_old_comments_async (const char *dir,
 
 
 static char *
-get_text (CommentData *data,
-	  xmlChar     *value)
+get_utf8_text (CommentData *data,
+	       xmlChar     *value)
 {
 	if (value == NULL)
 		return NULL;
-	if (! data->utf8_format)
+
+	if (data->utf8_format) 
 		return g_strdup (value);
 	else
-		return g_locale_from_utf8 (value, -1, NULL, NULL, NULL);
+		return g_locale_to_utf8 (value, -1, NULL, NULL, NULL);
 }
 
 
@@ -361,18 +447,18 @@ get_keywords (CommentData *data,
 	      xmlChar     *utf8_value)
               
 {
-        int       n, i;
-        xmlChar  *c, *sub_str;
-        gboolean  done;
 	char     *value;
+        int       n;
+        xmlChar  *p, *keyword;
+        gboolean  done;
 
-        if ((utf8_value == NULL) || (*utf8_value == 0)) {
+	if ((utf8_value == NULL) || (*utf8_value == 0)) {
 		data->keywords_n = 0;
 		data->keywords = NULL;
-                return;
+		return;
 	}
 
-	value = get_text (data, utf8_value);
+	value = get_utf8_text (data, utf8_value);
 
 	if (value == NULL) {
 		data->keywords_n = 0;
@@ -380,27 +466,30 @@ get_keywords (CommentData *data,
 		return;
 	}
 
-        n = 1;
-        for (c = value; *c != 0; c++)
-                if (*c == ',')
-                        n++;
-        data->keywords_n = n;
-        data->keywords = g_new0 (char*, n + 1);
+	n = 1;
+	for (p = value; *p != 0; p = g_utf8_next_char (p)) {
+		gunichar ch = g_utf8_get_char (p);
+		if (ch == ',')
+			n++;
+	}
+
+	data->keywords_n = n;
+	data->keywords = g_new0 (char*, n + 1);
 	data->keywords[n] = NULL;
 
-        i = 0;
-        sub_str = c = value;
-        do {
-                done = (*c == 0); 
+        n = 0;
+	keyword = p = value;
+	do {
+		gunichar ch = g_utf8_get_char (p);
 
-                if (*c == ',')
-                        *c = 0;
-                if (*c == 0) {
-			data->keywords[i++] = g_strdup (sub_str);
-			sub_str = c + 1;
-                }
-                c++;
-        } while (! done);
+		done = (*p == 0); 
+
+		if ((ch == ',') || (*p == 0)) {
+			data->keywords[n++] = g_strndup (keyword, p - keyword);
+			keyword = p = g_utf8_next_char (p);
+		} else
+			p = g_utf8_next_char (p);
+	} while (! done);
 
 	g_free (value);
 }
@@ -416,10 +505,19 @@ comments_load_comment (const char *filename)
         xmlChar     *value;
 	xmlChar     *format;
 
-	comment_file = comments_get_comment_filename (filename);
+	if (filename == NULL)
+		return NULL;
+
+	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
 	if (! path_is_file (comment_file)) {
 		g_free (comment_file);
-		return NULL;
+
+		comment_file = comments_get_comment_filename (filename, FALSE, TRUE);
+		if (! path_is_file (comment_file)) {
+			g_free (comment_file);
+
+			return NULL;
+		}
 	}
 
         doc = xmlParseFile (comment_file);
@@ -449,18 +547,18 @@ comments_load_comment (const char *filename)
                 value = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 
                 if (strcmp (node->name, PLACE_TAG) == 0) 
-			data->place = get_text (data, value);
+			data->place = get_utf8_text (data, value);
 		else if (strcmp (node->name, NOTE_TAG) == 0) 
-			data->comment = get_text (data, value);
+			data->comment = get_utf8_text (data, value);
 		else if (strcmp (node->name, KEYWORDS_TAG) == 0) 
-                        get_keywords (data, value);
+			get_keywords (data, value);
 		else if (strcmp (node->name, TIME_TAG) == 0) {
-                        if (value != NULL)
+			if (value != NULL)
 				data->time = atol (value);
 		}
 		
-                if (value)
-                        xmlFree (value);
+		if (value)
+			xmlFree (value);
         }
 
         xmlFreeDoc (doc);
@@ -480,10 +578,6 @@ save_comment (const char  *filename,
 	char        *time_str;
 	char        *keywords_str;
 	char        *dest_dir;
-	char        *place_utf8 = NULL;
-	char        *time_utf8 = NULL;
-	char        *keywords_utf8 = NULL;
-	char        *comment_utf8 = NULL;
 
 	if (comment_data_is_void (data)) {
 		comment_delete (filename);
@@ -501,16 +595,6 @@ save_comment (const char  *filename,
 	} else
 		keywords_str = g_strdup ("");
 
-	if (data->place != NULL)
-		place_utf8 = g_locale_to_utf8 (data->place, -1, NULL, NULL, NULL);
-	time_utf8 = g_locale_to_utf8 (time_str, -1, NULL, NULL, NULL);
-	if (data->comment != NULL)
-		comment_utf8 = g_locale_to_utf8 (data->comment, -1, NULL, NULL, NULL);
-	keywords_utf8 = g_locale_to_utf8 (keywords_str, -1, NULL, NULL, NULL);
-
-	g_free (time_str);
-	g_free (keywords_str);
-
 	/* Create the xml tree. */
 
 	doc = xmlNewDoc ("1.0");
@@ -519,19 +603,17 @@ save_comment (const char  *filename,
 	xmlSetProp (doc->xmlRootNode, FORMAT_TAG, FORMAT_VER);
 
 	tree = doc->xmlRootNode;
-	subtree = xmlNewChild (tree, NULL, PLACE_TAG, place_utf8);
-	subtree = xmlNewChild (tree, NULL, TIME_TAG, time_utf8);
-	subtree = xmlNewChild (tree, NULL, NOTE_TAG, comment_utf8);
-	subtree = xmlNewChild (tree, NULL, KEYWORDS_TAG, keywords_utf8);
+	subtree = xmlNewChild (tree, NULL, PLACE_TAG,    data->place);
+	subtree = xmlNewChild (tree, NULL, TIME_TAG,     time_str);
+	subtree = xmlNewChild (tree, NULL, NOTE_TAG,     data->comment);
+	subtree = xmlNewChild (tree, NULL, KEYWORDS_TAG, keywords_str);
 
-	g_free (place_utf8);
-	g_free (time_utf8);
-	g_free (comment_utf8);
-	g_free (keywords_utf8);
+	g_free (time_str);
+	g_free (keywords_str);
 
 	/* Write to disk. */
 
-	comment_file = comments_get_comment_filename (filename);
+	comment_file = comments_get_comment_filename (filename, TRUE, TRUE);
 	dest_dir = remove_level_from_path (comment_file);
 	if (ensure_dir_exists (dest_dir, 0700)) {
 		xmlSetDocCompressMode (doc, 3);
@@ -554,9 +636,12 @@ comments_save_comment (const char  *filename,
 
 	if (new_data == NULL) {
 		CommentData *data_without_categories;
+
 		data_without_categories = comment_data_dup (data);
 		comment_data_free_keywords (data_without_categories);
 		save_comment (filename, data_without_categories);
+		comment_data_free (data_without_categories);
+
 		return;
 	}
 
@@ -569,6 +654,7 @@ comments_save_comment (const char  *filename,
 		new_data->comment = g_strdup (data->comment);
 
 	save_comment (filename, new_data);
+	comment_data_free (new_data);
 }
 
 
@@ -616,9 +702,12 @@ comments_save_categories (const char  *filename,
 
 	if (new_data == NULL) {
 		CommentData *data_without_comment;
+
 		data_without_comment = comment_data_dup (data);
 		comment_data_free_comment (data_without_comment);
 		save_comment (filename, data_without_comment);
+		comment_data_free (data_without_comment);
+
 		return;
 	}
 
@@ -637,6 +726,7 @@ comments_save_categories (const char  *filename,
 	}
 
 	save_comment (filename, new_data);
+	comment_data_free (new_data);
 }
 
 
@@ -709,7 +799,7 @@ comment_data_remove_keyword (CommentData *data,
 		return;
 
 	for (i = 0; i < data->keywords_n; i++) 
-		if (strcmp (data->keywords[i], keyword) == 0) {
+		if (g_utf8_collate (data->keywords[i], keyword) == 0) {
 			found = TRUE;
 			break;
 		}
@@ -737,13 +827,13 @@ void
 comment_data_add_keyword (CommentData *data,
 			  const char  *keyword)
 {
-	int      i;
+	int i;
 
 	if (keyword == NULL)
 		return;
 
  	for (i = 0; i < data->keywords_n; i++) 
-		if (strcmp (data->keywords[i], keyword) == 0) 
+		if (g_utf8_collate (data->keywords[i], keyword) == 0) 
 			return;
 	
 	data->keywords_n++;
