@@ -37,6 +37,7 @@
 #define  PREF_MIGRATE_DIRECTORIES    "/apps/gthumb/general/migrate_directories"
 
 #define  PREF_SHOW_HIDDEN_FILES      "/apps/gthumb/browser/show_hidden_files"
+#define  PREF_SHOW_FILENAMES         "/apps/gthumb/browser/show_filenames"
 #define  PREF_SHOW_COMMENTS          "/apps/gthumb/browser/show_comments"
 #define  PREF_SHOW_THUMBNAILS        "/apps/gthumb/browser/show_thumbnails"
 #define  PREF_FAST_FILE_TYPE         "/apps/gthumb/browser/fast_file_type"
@@ -48,6 +49,9 @@
 #define  PREF_ARRANGE_IMAGES         "/apps/gthumb/browser/arrange_images"
 #define  PREF_SORT_IMAGES            "/apps/gthumb/browser/sort_images"
 #define  PREF_VIEW_AS                "/apps/gthumb/browser/view_as"
+#define  PREF_SHOW_PREVIEW           "/apps/gthumb/browser/show_preview"
+#define  PREF_PREVIEW_CONTENT        "/apps/gthumb/browser/preview_content"
+#define  PREF_SHOW_IMAGE_DATA        "/apps/gthumb/browser/show_image_data"
 
 #define  PREF_ZOOM_QUALITY           "/apps/gthumb/viewer/zoom_quality"
 #define  PREF_ZOOM_CHANGE            "/apps/gthumb/viewer/zoom_change"
@@ -139,12 +143,25 @@
 #define  PREF_WEB_ALBUM_DESTINATION   "/apps/gthumb/dialogs/web_album/destination"
 #define  PREF_WEB_ALBUM_INDEX_FILE    "/apps/gthumb/dialogs/web_album/index_file"
 #define  PREF_WEB_ALBUM_COPY_IMAGES   "/apps/gthumb/dialogs/web_album/copy_images"
+#define  PREF_WEB_ALBUM_RESIZE_IMAGES "/apps/gthumb/dialogs/web_album/resize_images"
+#define  PREF_WEB_ALBUM_RESIZE_WIDTH  "/apps/gthumb/dialogs/web_album/resize_width"
+#define  PREF_WEB_ALBUM_RESIZE_HEIGHT "/apps/gthumb/dialogs/web_album/resize_height"
 #define  PREF_WEB_ALBUM_ROWS          "/apps/gthumb/dialogs/web_album/rows"
 #define  PREF_WEB_ALBUM_COLUMNS       "/apps/gthumb/dialogs/web_album/columns"
 #define  PREF_WEB_ALBUM_SORT          "/apps/gthumb/dialogs/web_album/sort_by"
 #define  PREF_WEB_ALBUM_REVERSE       "/apps/gthumb/dialogs/web_album/reverse_order"
 #define  PREF_WEB_ALBUM_TITLE         "/apps/gthumb/dialogs/web_album/title"
 #define  PREF_WEB_ALBUM_THEME         "/apps/gthumb/dialogs/web_album/theme"
+#define  PREF_WEB_ALBUM_INDEX_CAPTION "/apps/gthumb/dialogs/web_album/index_caption"
+#define  PREF_WEB_ALBUM_IMAGE_CAPTION "/apps/gthumb/dialogs/web_album/image_caption"
+#define  PREF_SEARCH_RECURSIVE        "/apps/gthumb/dialogs/search/recursive"
+#define  PREF_PRINT_PAPER_SIZE        "/apps/gthumb/dialogs/print/paper_size"
+#define  PREF_PRINT_PAPER_ORIENTATION "/apps/gthumb/dialogs/print/paper_orientation"
+#define  PREF_PRINT_INCLUDE_COMMENT   "/apps/gthumb/dialogs/print/include_comment"
+#define  PREF_PRINT_COMMENT_FONT      "/apps/gthumb/dialogs/print/comment_font"
+#define  PREF_PRINT_PAPER_WIDTH       "/apps/gthumb/dialogs/print/paper_width"
+#define  PREF_PRINT_PAPER_HEIGHT      "/apps/gthumb/dialogs/print/paper_height"
+#define  PREF_PRINT_PAPER_UNIT        "/apps/gthumb/dialogs/print/paper_unit"
 
 
 typedef struct {
@@ -165,9 +182,9 @@ typedef struct {
 } Preferences;
 
 
-void            preferences_init                      ();
+void            preferences_init                      (void);
 
-void            preferences_release                   ();
+void            preferences_release                   (void);
 
 const  char    *pref_util_get_file_location           (const char *location);
 
@@ -194,71 +211,71 @@ const char     *pref_util_get_hex_value               (guint16     r,
 
 guint32         pref_util_get_int_value               (const char *hex);
 
-GthClickPolicy  pref_get_real_click_policy            ();
+GthClickPolicy  pref_get_real_click_policy            (void);
 
 /* ------- */
 
-GthClickPolicy     pref_get_click_policy           ();
+GthClickPolicy     pref_get_click_policy           (void);
 
 void               pref_set_click_policy           (GthClickPolicy value);
 
-GthSortMethod      pref_get_arrange_type           ();
+GthSortMethod      pref_get_arrange_type           (void);
 
 void               pref_set_arrange_type           (GthSortMethod value);
 
-GtkSortType        pref_get_sort_order             ();
+GtkSortType        pref_get_sort_order             (void);
 
 void               pref_set_sort_order             (GtkSortType value);
 
-GthZoomQuality     pref_get_zoom_quality           ();
+GthZoomQuality     pref_get_zoom_quality           (void);
 
 void               pref_set_zoom_quality           (GthZoomQuality value);
 
-GthZoomChange      pref_get_zoom_change            ();
+GthZoomChange      pref_get_zoom_change            (void);
 
 void               pref_set_zoom_change            (GthZoomChange value);
 
-GthTranspType      pref_get_transp_type            ();
+GthTranspType      pref_get_transp_type            (void);
 
 void               pref_set_transp_type            (GthTranspType value);
 
-GthCheckType       pref_get_check_type             ();
+GthCheckType       pref_get_check_type             (void);
 
 void               pref_set_check_type             (GthCheckType value);
 
-GthCheckSize       pref_get_check_size             ();
+GthCheckSize       pref_get_check_size             (void);
 
 void               pref_set_check_size             (GthCheckSize value);
 
-GthDirectionType   pref_get_slideshow_direction    ();
+GthDirectionType   pref_get_slideshow_direction    (void);
 
 void               pref_set_slideshow_direction    (GthDirectionType value);
 
-GthToolbarStyle    pref_get_toolbar_style          ();
+GthToolbarStyle    pref_get_toolbar_style          (void);
 
 void               pref_set_toolbar_style          (GthToolbarStyle value);
 
-GthSortMethod      pref_get_exp_arrange_type       ();
+GthSortMethod      pref_get_exp_arrange_type       (void);
 
 void               pref_set_exp_arrange_type       (GthSortMethod value);
 
-GtkSortType        pref_get_exp_sort_order         ();
+GtkSortType        pref_get_exp_sort_order         (void);
 
 void               pref_set_exp_sort_order         (GtkSortType value);
 
-GthFrameStyle      pref_get_exporter_frame_style   ();
+GthFrameStyle      pref_get_exporter_frame_style   (void);
 
 void               pref_set_exporter_frame_style   (GthFrameStyle value);
 
-GthOverwriteMode   pref_get_convert_overwrite_mode ();
+GthOverwriteMode   pref_get_convert_overwrite_mode (void);
 
 void               pref_set_convert_overwrite_mode (GthOverwriteMode value);
 
-GthSortMethod      pref_get_rename_sort_order      ();
+GthSortMethod      pref_get_rename_sort_order      (void);
 
 void               pref_set_rename_sort_order      (GthSortMethod value);
 
-GthSortMethod      pref_get_web_album_sort_order   ();
+GthSortMethod      pref_get_web_album_sort_order   (void);
 
 void               pref_set_web_album_sort_order   (GthSortMethod value);
 
@@ -266,4 +283,14 @@ GthViewAs          pref_get_view_as                (void);
 
 void               pref_set_view_as                (GthViewAs value);
 
+void               pref_set_preview_content        (GthPreviewContent value);
+
+GthPreviewContent  pref_get_preview_content        (void);
+
+GthViewMode        pref_get_view_mode              (void);
+
+void               pref_set_print_unit             (GthPrintUnit value);
+
+GthPrintUnit       pref_get_print_unit             (void);
+
 #endif /* PREFERENCES_H */
