@@ -123,7 +123,7 @@ file_new_window_command_impl (BonoboUIComponent *uic,
 	pref_set_check_type (image_viewer_get_check_type (viewer));
 	pref_set_check_size (image_viewer_get_check_size (viewer));
 
-	eel_gconf_set_boolean (PREF_UI_IMAGE_PANE_VISIBLE, window->image_preview_visible);
+	eel_gconf_set_boolean (PREF_UI_IMAGE_PANE_VISIBLE, window->preview_visible);
 
 	/**/
 
@@ -177,6 +177,16 @@ image_open_with_command_impl (BonoboUIComponent *uic,
 }
 
 
+static void
+image_saved_func (char     *filename,
+		  gpointer  data)
+{
+	GThumbWindow *window = data;
+	if (filename != NULL) 
+		window_load_image (window, filename);
+}
+
+
 void 
 file_save_command_impl (BonoboUIComponent *uic, 
 			gpointer           user_data, 
@@ -195,7 +205,9 @@ file_save_command_impl (BonoboUIComponent *uic,
 
 	dlg_save_image (GTK_WINDOW (window->app), 
 			current_folder,
-			image_viewer_get_current_pixbuf (IMAGE_VIEWER (window->viewer)));
+			image_viewer_get_current_pixbuf (IMAGE_VIEWER (window->viewer)),
+			image_saved_func,
+			window);
 
 	g_free (current_folder);
 }
@@ -424,14 +436,33 @@ duplicate_file (GThumbWindow *window,
 		comment_copy (old_path, new_path);
 
 	} else {
-		char *utf8_path;
+		char      *utf8_path;
+		char      *msg;
+		GtkWidget *d;
+		int        r;
 
 		utf8_path = g_locale_to_utf8 (old_path, -1, NULL, NULL, NULL);
-		_gtk_error_dialog_run (GTK_WINDOW (window->app),
-				       _("Could not duplicate the image \"%s\": %s"),
+		msg = g_strdup_printf (_("Could not duplicate the image \"%s\": %s"),
 				       utf8_path,
 				       errno_to_string ());
 		g_free (utf8_path);
+
+		d = _gtk_yesno_dialog_new (GTK_WINDOW (window->app),
+					   GTK_DIALOG_MODAL,
+					   msg,
+					   _("Stop"),
+					   _("Continue"));
+		g_free (msg);
+
+		r = gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (GTK_WIDGET (d));
+
+		if (r != GTK_RESPONSE_YES) {
+			g_free (new_path);
+			g_free (new_name);
+			return;
+		}
+					   
 	}
 
 	g_free (new_name);
@@ -1034,10 +1065,13 @@ folder_rename (GThumbWindow *window,
 
 		/* Comment cache. */
 
-		old_cache_path = comments_get_comment_dir (old_path);
-		new_cache_path = comments_get_comment_dir (new_path);
+		old_cache_path = comments_get_comment_dir (old_path, TRUE, TRUE);
+		if (! path_is_dir (old_cache_path)) {
+			g_free (old_cache_path);
+			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
+		}
+		new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
 		rename (old_cache_path, new_cache_path);
-
 		g_free (old_cache_path);
 		g_free (new_cache_path);
 
@@ -1356,8 +1390,12 @@ folder_copy__response_cb (GObject *object,
 
 		/* Comment cache. */
 
-		old_cache_path = comments_get_comment_dir (old_path);
-		new_cache_path = comments_get_comment_dir (new_path);
+		old_cache_path = comments_get_comment_dir (old_path, TRUE, TRUE);
+		if (! path_is_dir (old_cache_path)) {
+			g_free (old_cache_path);
+			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
+		}
+		new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
 
 		if (path_is_dir (old_cache_path)) {
 			char *parent_dir;
@@ -1912,7 +1950,7 @@ alter_image_rotate_command_impl (BonoboUIComponent *uic,
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_image_set_modified (window, TRUE);
 }
 
 
@@ -1931,7 +1969,7 @@ alter_image_rotate_cc_command_impl (BonoboUIComponent *uic,
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_image_set_modified (window, TRUE);
 }
 
 
@@ -1950,7 +1988,7 @@ alter_image_flip_command_impl (BonoboUIComponent *uic,
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_image_set_modified (window, TRUE);
 }
 
 
@@ -1969,7 +2007,7 @@ alter_image_mirror_command_impl (BonoboUIComponent *uic,
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
 	g_object_unref (dest_pixbuf);
 
-	window_image_modified (window, TRUE);
+	window_image_set_modified (window, TRUE);
 }
 
 
@@ -2057,6 +2095,48 @@ alter_image_adjust_levels_command_impl (BonoboUIComponent *uic,
 }
 
 
+void 
+alter_image_stretch_contrast_command_impl (BonoboUIComponent *uic, 
+					   gpointer           user_data, 
+					   const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	GdkPixbuf    *src_pixbuf;
+	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
+
+	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
+	pixop = _gdk_pixbuf_stretch_contrast (dest_pixbuf, dest_pixbuf);
+	g_object_unref (dest_pixbuf);
+
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
+}
+
+
+void 
+alter_image_normalize_contrast_command_impl (BonoboUIComponent *uic, 
+					     gpointer           user_data, 
+					     const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
+	GdkPixbuf    *src_pixbuf;
+	GdkPixbuf    *dest_pixbuf;
+	GthPixbufOp  *pixop;
+
+	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	dest_pixbuf = gdk_pixbuf_copy (src_pixbuf);
+	pixop = _gdk_pixbuf_normalize_contrast (dest_pixbuf, dest_pixbuf);
+	g_object_unref (dest_pixbuf);
+
+	window_exec_pixbuf_op (window, pixop);
+	g_object_unref (pixop);
+}
+
+
 void 
 alter_image_posterize_command_impl (BonoboUIComponent *uic, 
 				    gpointer           user_data, 
@@ -2145,11 +2225,6 @@ view_zoom_fit_command_impl (BonoboUIComponent *uic,
 	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 
-	if (window->freeze_toggle_handler > 0) {
-		window->freeze_toggle_handler--;
-		return;
-	}
-
 	if (viewer->zoom_fit) 
 		viewer->zoom_fit = FALSE;
 	else 
@@ -2225,11 +2300,6 @@ view_fullscreen_command_impl (BonoboUIComponent *uic,
 {
 	GThumbWindow *window = user_data;
 
-	if (window->freeze_toggle_handler > 0) {
-		window->freeze_toggle_handler--;
-		return;
-	}
-
 	if (! window->fullscreen)
 		fullscreen_start (fullscreen, window);
 	else
@@ -2741,7 +2811,7 @@ help_about_command_impl (BonoboUIComponent *uic,
 	logo = gdk_pixbuf_new_from_file (PIXMAPSDIR "/gthumb.png", NULL);
 	about = gnome_about_new (_("gThumb"), 
 				 VERSION,
-				 _("Copyright (C) 2001 Free Software Foundation, Inc."),
+				 "Copyright \xc2\xa9 2001-2003 Free Software Foundation, Inc.",
 				 _("An image viewer and browser for GNOME."),
 				 authors,
 				 documenters,
