@@ -29,6 +29,8 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 
+#include "async-pixbuf-ops.h"
+#include "gth-pixbuf-op.h"
 #include "gthumb-window.h"
 #include "pixbuf-utils.h"
 
@@ -45,6 +47,7 @@ typedef struct {
 	GdkPixbuf    *orig_pixbuf;
 	GdkPixbuf    *new_pixbuf;
 
+	gboolean      original_modified;
 	gboolean      modified;
 
 	GladeXML     *gui;
@@ -69,26 +72,45 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
+static gboolean
+preview_done_cb (GthPixbufOp *pixop,
+		 gboolean     completed,
+		 DialogData  *data)
+{
+	gtk_widget_queue_draw (data->bc_preview_image);
+	g_object_unref (pixop);
+	return FALSE;
+}
+
+
 static void
 apply_changes (DialogData *data, 
 	       GdkPixbuf  *src, 
 	       GdkPixbuf  *dest, 
 	       gboolean    preview)
 {
-	double brightness;
-	double contrast;
+	double       brightness;
+	double       contrast;
+	GthPixbufOp *pixop;
 
 	brightness = gtk_range_get_value (GTK_RANGE (data->bc_brightness_hscale));
 	contrast   = gtk_range_get_value (GTK_RANGE (data->bc_contrast_hscale));
 
-	_gdk_pixbuf_brightness_contrast (src,
-					 dest,
-					 brightness / 127.0,
-					 contrast / 127.0);
+	pixop = _gdk_pixbuf_brightness_contrast (src,
+						 dest,
+						 brightness / 127.0,
+						 contrast / 127.0);
 
-	if (preview)
-		gtk_image_set_from_pixbuf (GTK_IMAGE (data->bc_preview_image),
-					   dest);
+	if (preview) {
+		g_signal_connect (G_OBJECT (pixop),
+				  "done",
+				  G_CALLBACK (preview_done_cb),
+				  data);
+		gth_pixbuf_op_start (pixop);
+	} else {
+		window_exec_pixbuf_op (data->window, pixop);
+		g_object_unref (pixop);
+	}		
 }
 
 
@@ -99,7 +121,7 @@ ok_cb (GtkWidget  *widget,
 {
 	apply_changes (data, data->image, data->image, FALSE);
 	image_viewer_set_pixbuf (data->viewer, data->image);
-	window_image_modified (data->window, TRUE);
+	window_image_set_modified (data->window, TRUE);
 	gtk_widget_destroy (data->dialog);
 }
 
@@ -109,8 +131,10 @@ static void
 cancel_cb (GtkWidget  *widget, 
 	   DialogData *data)
 {
-	if (data->modified)
+	if (data->modified) {
 		image_viewer_set_pixbuf (data->viewer, data->image);
+		window_image_set_modified (data->window, data->original_modified);
+	}
 	gtk_widget_destroy (data->dialog);
 }
 
@@ -124,7 +148,6 @@ preview_cb (GtkWidget  *widget,
 
 	preview = gdk_pixbuf_copy (data->image);
 	apply_changes (data, preview, preview, FALSE);
-	image_viewer_set_pixbuf (data->viewer, preview);
 	g_object_unref (preview);
 
 	data->modified = TRUE;
@@ -198,6 +221,7 @@ dlg_brightness_contrast (GThumbWindow *window)
 
 	data = g_new0 (DialogData, 1);
 	data->window = window;
+	data->original_modified = window_image_get_modified (window);
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE, NULL,
 				   NULL);
 
