@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -168,6 +168,7 @@ dlg_show_error (GThumbWindow *window,
 				     GTK_DIALOG_MODAL,
 				     GTK_STOCK_DIALOG_ERROR,
 				     utf8_msg,
+				     NULL,
 				     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
 				     NULL);
 	g_free (msg);
@@ -678,7 +679,7 @@ create_overwrite_dialog (GThumbWindow      *window,
 
 	viewer = IMAGE_VIEWER (old_image_viewer);
 	image_viewer_size             (viewer, PREVIEW_SIZE, PREVIEW_SIZE);
-	image_viewer_set_zoom_quality (viewer, pref_get_zoom_change ());
+	image_viewer_set_zoom_quality (viewer, pref_get_zoom_quality ());
 	image_viewer_set_check_type   (viewer, image_viewer_get_check_type (IMAGE_VIEWER (window->viewer)));
 	image_viewer_set_check_size   (viewer, image_viewer_get_check_size (IMAGE_VIEWER (window->viewer)));
 	image_viewer_set_transp_type  (viewer, image_viewer_get_transp_type (IMAGE_VIEWER (window->viewer)));
@@ -1153,6 +1154,7 @@ continue_or_abort_dialog (FileCopyData   *fcdata,
 					     GTK_DIALOG_MODAL,
 					     GTK_STOCK_DIALOG_ERROR,
 					     message,
+					     NULL,
 					     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 					     NULL);
 	else 
@@ -1437,8 +1439,13 @@ copy_next_file (FileCopyData *fcdata)
 					 file_name_from_path (src_file), 
 					 NULL);
 		
-		src_cache_file = comments_get_comment_filename (src_file);
-		dest_cache_file = comments_get_comment_filename (dest_file);
+		src_cache_file = comments_get_comment_filename (src_file, TRUE, TRUE);
+		if (! path_is_file (src_cache_file)) {
+			g_free (src_cache_file);
+			src_cache_file = comments_get_comment_filename (src_file, FALSE, TRUE);
+		}
+
+		dest_cache_file = comments_get_comment_filename (dest_file, TRUE, TRUE);
 
 		src_cache_dir = remove_level_from_path (src_cache_file);
 
@@ -2056,8 +2063,13 @@ folder_copy (GThumbWindow   *window,
 		char *src_cache;
 		char *dest_cache;
 
-		src_cache = comments_get_comment_dir (src_path);
-		dest_cache = comments_get_comment_dir (dest_path);
+		src_cache = comments_get_comment_dir (src_path, TRUE, TRUE);
+		if (! path_is_dir (src_cache)) {
+			g_free (src_cache);
+			src_cache = comments_get_comment_dir (src_path, FALSE, TRUE);
+		}
+
+		dest_cache = comments_get_comment_dir (dest_path, TRUE, TRUE);
 
 		if (path_is_dir (src_cache)) {
 			char *parent_dir = remove_level_from_path (dest_cache);
@@ -2298,6 +2310,7 @@ copy_item__continue1 (GnomeVFSResult result,
 						     GTK_DIALOG_MODAL,
 						     GTK_STOCK_DIALOG_ERROR,
 						     message,
+						     NULL,
 						     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 						     NULL);
 		else 
