@@ -44,8 +44,13 @@
 #include "gthumb-window.h"
 #include "glib-utils.h"
 
-int           sort_method_to_idx[] = { -1, 0, 1, 2, 3 };
-GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME, GTH_SORT_METHOD_BY_PATH, GTH_SORT_METHOD_BY_SIZE, GTH_SORT_METHOD_BY_TIME };
+static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3 };
+static GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME, 
+					      GTH_SORT_METHOD_BY_PATH, 
+					      GTH_SORT_METHOD_BY_SIZE, 
+					      GTH_SORT_METHOD_BY_TIME };
+static int           idx_to_resize_width[] = { 320, 640, 800, 1024, 1280 };
+static int           idx_to_resize_height[] = { 200, 480, 600, 768, 960 };
 
 
 #define GLADE_EXPORTER_FILE "gthumb_web_exporter.glade"
@@ -68,6 +73,10 @@ typedef struct {
 	GtkWidget          *dest_fileentry_entry;
 	GtkWidget          *wa_index_file_entry;
 	GtkWidget          *wa_copy_images_checkbutton;
+	GtkWidget          *wa_resize_images_checkbutton;
+	GtkWidget          *wa_resize_images_optionmenu;
+	GtkWidget          *wa_resize_images_hbox;
+	GtkWidget          *wa_resize_images_options_hbox;
 
 	GtkWidget          *wa_rows_spinbutton;
 	GtkWidget          *wa_cols_spinbutton;
@@ -80,15 +89,6 @@ typedef struct {
 
 	/**/
 
-	GtkWidget          *wat_dialog;
-	GtkWidget          *wat_theme_treeview;
-	GtkWidget          *wat_ok_button;
-	GtkWidget          *wat_cancel_button;
-	GtkWidget          *wat_install_button;
-	GtkWidget          *wat_go_to_folder_button;
-
-	/**/
-
 	CatalogWebExporter *exporter;
 } DialogData;
 
@@ -99,7 +99,7 @@ destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	g_object_unref (data->gui);
-	if (data->exporter)
+	if (data->exporter != NULL)
 		g_object_unref (data->exporter);
 	g_free (data);
 }
@@ -126,6 +126,12 @@ export (GtkWidget  *widget,
 
 	eel_gconf_set_boolean (PREF_WEB_ALBUM_COPY_IMAGES, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_copy_images_checkbutton)));
 
+	eel_gconf_set_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_resize_images_checkbutton)));
+
+	eel_gconf_set_integer (PREF_WEB_ALBUM_RESIZE_WIDTH, idx_to_resize_width[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))]);
+
+	eel_gconf_set_integer (PREF_WEB_ALBUM_RESIZE_HEIGHT, idx_to_resize_height[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))]);
+
 	eel_gconf_set_integer (PREF_WEB_ALBUM_ROWS, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->wa_rows_spinbutton)));
 
 	eel_gconf_set_integer (PREF_WEB_ALBUM_COLUMNS, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->wa_cols_spinbutton)));
@@ -156,6 +162,11 @@ export (GtkWidget  *widget,
 
 	catalog_web_exporter_set_copy_images (exporter, eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES));
 
+	catalog_web_exporter_set_resize_images (exporter, 
+						eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES),
+						idx_to_resize_width[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))],
+						idx_to_resize_height[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))]);
+
 	catalog_web_exporter_set_row_col (exporter, eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS));
 	
 	catalog_web_exporter_set_sorted (exporter, pref_get_web_album_sort_order (), eel_gconf_get_boolean (PREF_WEB_ALBUM_REVERSE));
@@ -216,6 +227,40 @@ export_start_copying (GtkObject  *object,
 static void show_album_theme_cb (GtkWidget *widget, DialogData *data);
 
 
+static int
+get_idx_from_resize_width (int width)
+{
+	if (width == 320)
+		return 0;
+	else if (width == 640)
+		return 1;
+	else if (width == 800)
+		return 2;
+	else if (width == 1024)
+		return 3;
+	else if (width == 1280)
+		return 4;
+	else
+		return 1;
+}
+
+
+static void
+copy_image_toggled_cb (GtkToggleButton *button,
+		       DialogData      *data)
+{
+	gtk_widget_set_sensitive (data->wa_resize_images_hbox, gtk_toggle_button_get_active (button));
+}
+
+
+static void
+resize_image_toggled_cb (GtkToggleButton *button,
+			 DialogData      *data)
+{
+	gtk_widget_set_sensitive (data->wa_resize_images_options_hbox, gtk_toggle_button_get_active (button));
+}
+
+
 /* create the main dialog. */
 void
 dlg_web_exporter (GThumbWindow *window)
@@ -256,6 +301,11 @@ dlg_web_exporter (GThumbWindow *window)
 	data->wa_index_file_entry = glade_xml_get_widget (data->gui, "wa_index_file_entry");
 	data->wa_copy_images_checkbutton = glade_xml_get_widget (data->gui, "wa_copy_images_checkbutton");
 
+	data->wa_resize_images_checkbutton = glade_xml_get_widget (data->gui, "wa_resize_images_checkbutton");
+	data->wa_resize_images_optionmenu = glade_xml_get_widget (data->gui, "wa_resize_images_optionmenu");
+	data->wa_resize_images_hbox = glade_xml_get_widget (data->gui, "wa_resize_images_hbox");
+	data->wa_resize_images_options_hbox = glade_xml_get_widget (data->gui, "wa_resize_images_options_hbox");
+
 	data->wa_rows_spinbutton = glade_xml_get_widget (data->gui, "wa_rows_spinbutton");
 	data->wa_cols_spinbutton = glade_xml_get_widget (data->gui, "wa_cols_spinbutton");
 	data->wa_sort_images_optionmenu = glade_xml_get_widget (data->gui, "wa_sort_images_optionmenu");
@@ -277,6 +327,46 @@ dlg_web_exporter (GThumbWindow *window)
 
 	data->dest_fileentry_entry = gnome_entry_gtk_entry (GNOME_ENTRY (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (data->wa_dest_fileentry))));
 
+	/* Set widgets data. */
+
+	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_DESTINATION);
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->dest_fileentry_entry),
+				    ((svalue == NULL) || (*svalue == 0)) ? g_get_home_dir() : svalue);
+	g_free (svalue);
+
+	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_INDEX_FILE);
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_index_file_entry), svalue);
+	g_free (svalue);
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_copy_images_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES));
+
+	gtk_widget_set_sensitive (data->wa_resize_images_hbox, eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES));
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_resize_images_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES));
+
+	gtk_widget_set_sensitive (data->wa_resize_images_options_hbox, eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES));
+
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu), get_idx_from_resize_width (eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_WIDTH)));
+
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_rows_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS));
+
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_cols_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS));
+
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_sort_images_optionmenu), sort_method_to_idx [pref_get_web_album_sort_order ()]);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_reverse_order_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_REVERSE));
+
+	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_TITLE);
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_title_entry), svalue);
+	g_free (svalue);
+
+	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_THEME);
+	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_theme_entry), svalue);
+	g_free (svalue);
+
+	catalog_web_exporter_set_index_caption (data->exporter, eel_gconf_get_integer (PREF_WEB_ALBUM_INDEX_CAPTION));
+	catalog_web_exporter_set_image_caption (data->exporter, eel_gconf_get_integer (PREF_WEB_ALBUM_IMAGE_CAPTION));
+
 	/* Signals. */
 
 	g_signal_connect (G_OBJECT (data->dialog), 
@@ -297,6 +387,15 @@ dlg_web_exporter (GThumbWindow *window)
 			  G_CALLBACK (show_album_theme_cb),
 			  data);
 
+	g_signal_connect (G_OBJECT (data->wa_copy_images_checkbutton), 
+			  "toggled",
+			  G_CALLBACK (copy_image_toggled_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->wa_resize_images_checkbutton), 
+			  "toggled",
+			  G_CALLBACK (resize_image_toggled_cb),
+			  data);
+
 	g_signal_connect (G_OBJECT (data->exporter), 
 			  "done",
 			  G_CALLBACK (export_done),
@@ -323,35 +422,6 @@ dlg_web_exporter (GThumbWindow *window)
 				  G_CALLBACK (catalog_web_exporter_interrupt),
 				  data->exporter);
 
-	/* Set widgets data. */
-
-	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_DESTINATION);
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->dest_fileentry_entry),
-				    ((svalue == NULL) || (*svalue == 0)) ? g_get_home_dir() : svalue);
-	g_free (svalue);
-
-	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_INDEX_FILE);
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_index_file_entry), svalue);
-	g_free (svalue);
-	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_copy_images_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES));
-
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_rows_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS));
-
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_cols_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS));
-
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_sort_images_optionmenu), sort_method_to_idx [pref_get_web_album_sort_order ()]);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_reverse_order_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_REVERSE));
-
-	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_TITLE);
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_title_entry), svalue);
-	g_free (svalue);
-
-	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_THEME);
-	_gtk_entry_set_locale_text (GTK_ENTRY (data->wa_theme_entry), svalue);
-	g_free (svalue);
-
 	/* Run dialog. */
 
 	gtk_widget_grab_focus (data->wa_dest_fileentry); 
@@ -377,6 +447,8 @@ typedef struct {
 	GtkWidget          *wat_cancel_button;
 	GtkWidget          *wat_install_button;
 	GtkWidget          *wat_go_to_folder_button;
+	GtkWidget          *wat_thumbnail_caption_button;
+	GtkWidget          *wat_image_caption_button;
 
 	GtkListStore       *list_store;
 } ThemeDialogData;
@@ -432,7 +504,9 @@ add_theme_dir (ThemeDialogData *tdata,
 	GList          *file_list = NULL;
 	GList          *scan;
 
+#ifdef DEBUG
 	g_print ("theme dir: %s\n", theme_dir);
+#endif
 
 	if (theme_dir != NULL)
 		result = gnome_vfs_directory_list_load (&file_list, 
@@ -491,18 +565,6 @@ load_themes (ThemeDialogData *tdata)
 }
 
 
-static void
-theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
-			      ThemeDialogData  *tdata)
-{
-	gboolean     theme_selected;
-	GtkTreeIter  iter;
-
-	theme_selected = gtk_tree_selection_get_selected (selection, NULL, &iter);
-	gtk_widget_set_sensitive (tdata->wat_ok_button, theme_selected);
-}
-
-
 static void
 theme_dialog__row_activated_cb (GtkTreeView       *tree_view,
 				GtkTreePath       *path,
@@ -631,6 +693,9 @@ theme_dialog__go_to_folder_clicked (GtkWidget       *widget,
 }
 
 
+static void show_thumbnail_caption_dialog_cb (GtkWidget  *widget,  ThemeDialogData *data);
+static void show_image_caption_dialog_cb     (GtkWidget  *widget,  ThemeDialogData *data);
+
 static void
 show_album_theme_cb (GtkWidget  *widget,
 		     DialogData *data)
@@ -659,6 +724,8 @@ show_album_theme_cb (GtkWidget  *widget,
 	tdata->wat_cancel_button = glade_xml_get_widget (tdata->gui, "wat_cancel_button");
 	tdata->wat_install_button = glade_xml_get_widget (tdata->gui, "wat_install_button");
 	tdata->wat_go_to_folder_button = glade_xml_get_widget (tdata->gui, "wat_go_to_folder_button");
+	tdata->wat_thumbnail_caption_button = glade_xml_get_widget (tdata->gui, "wat_thumbnail_caption_button");
+	tdata->wat_image_caption_button = glade_xml_get_widget (tdata->gui, "wat_image_caption_button");
 
 	/* Signals. */
 
@@ -674,10 +741,6 @@ show_album_theme_cb (GtkWidget  *widget,
 			  "clicked",
 			  G_CALLBACK (theme_dialog__ok_clicked),
 			  tdata);
-	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (tdata->wat_theme_treeview))),
-			  "changed",
-			  G_CALLBACK (theme_dialog__sel_changed_cb),
-			  tdata);
 	g_signal_connect (G_OBJECT (tdata->wat_theme_treeview),
 			  "row_activated",
 			  G_CALLBACK (theme_dialog__row_activated_cb),
@@ -691,6 +754,14 @@ show_album_theme_cb (GtkWidget  *widget,
 			  "clicked",
 			  G_CALLBACK (theme_dialog__go_to_folder_clicked),
 			  tdata);
+	g_signal_connect (G_OBJECT (tdata->wat_thumbnail_caption_button), 
+			  "clicked",
+			  G_CALLBACK (show_thumbnail_caption_dialog_cb),
+			  tdata);
+	g_signal_connect (G_OBJECT (tdata->wat_image_caption_button), 
+			  "clicked",
+			  G_CALLBACK (show_image_caption_dialog_cb),
+			  tdata);
 
 	/* Set widgets data. */
 
@@ -715,8 +786,6 @@ show_album_theme_cb (GtkWidget  *widget,
 
 	load_themes (tdata);
 
-	gtk_widget_set_sensitive (tdata->wat_ok_button, FALSE);
-
 	/* Run dialog. */
 
 	gtk_widget_grab_focus (tdata->wat_theme_treeview); 
@@ -725,3 +794,211 @@ show_album_theme_cb (GtkWidget  *widget,
 	gtk_window_set_modal (GTK_WINDOW (tdata->dialog), FALSE);
 	gtk_widget_show_all (tdata->dialog);
 }
+
+
+
+
+typedef struct {
+	DialogData         *data;
+	GThumbWindow       *window;
+
+	GladeXML           *gui;
+	GtkWidget          *dialog;
+
+	GtkWidget          *c_comment_checkbutton;
+	GtkWidget          *c_imagedim_checkbutton;
+	GtkWidget          *c_filename_checkbutton;
+	GtkWidget          *c_filesize_checkbutton;
+
+	GtkWidget          *c_exif_date_time_checkbutton;
+	GtkWidget          *c_exif_exposure_time_checkbutton;
+	GtkWidget          *c_exif_exposure_mode_checkbutton;
+	GtkWidget          *c_exif_flash_checkbutton;
+	GtkWidget          *c_exif_shutter_speed_checkbutton;
+	GtkWidget          *c_exif_aperture_value_checkbutton;
+	GtkWidget          *c_exif_focal_length_checkbutton;
+	GtkWidget          *c_exif_camera_model_checkbutton;
+
+	gboolean            thumbnail_caption;
+} CaptionDialogData;
+
+
+/* called when the dialog is closed. */
+static void
+caption_dialog_destroy_cb (GtkWidget         *widget, 
+			   CaptionDialogData *cdata)
+{
+	g_object_unref (cdata->gui);
+	g_free (cdata);
+}
+
+
+static void
+caption_dialog__ok_clicked (GtkWidget         *widget, 
+			    CaptionDialogData *cdata)
+{
+	const char       *gconf_key;
+	GthCaptionFields  caption = 0;
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton)))
+		caption |= GTH_CAPTION_COMMENT;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton)))
+		caption |= GTH_CAPTION_IMAGE_DIM;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_filename_checkbutton)))
+		caption |= GTH_CAPTION_FILE_NAME;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_filesize_checkbutton)))
+		caption |= GTH_CAPTION_FILE_SIZE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_date_time_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_DATE_TIME;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_exposure_time_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_EXPOSURE_TIME;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_exposure_mode_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_EXPOSURE_MODE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_flash_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_FLASH;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_shutter_speed_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_SHUTTER_SPEED;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_aperture_value_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_APERTURE_VALUE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_focal_length_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_FOCAL_LENGTH;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_exif_camera_model_checkbutton)))
+		caption |= GTH_CAPTION_EXIF_CAMERA_MODEL;
+
+	if (cdata->thumbnail_caption) {
+		catalog_web_exporter_set_index_caption (cdata->data->exporter, caption);
+		gconf_key = PREF_WEB_ALBUM_INDEX_CAPTION;
+	} else {
+		catalog_web_exporter_set_image_caption (cdata->data->exporter, caption);
+		gconf_key = PREF_WEB_ALBUM_IMAGE_CAPTION;
+	}
+
+	eel_gconf_set_integer (gconf_key, caption);
+
+	gtk_widget_destroy (cdata->dialog);
+}
+
+
+static void
+show_caption_dialog_cb (GtkWidget       *widget,
+			ThemeDialogData *tdata,
+			gboolean         thumbnail_caption)
+{
+	CaptionDialogData *cdata;
+	GtkWidget         *ok_button;
+	GtkWidget         *cancel_button;
+#ifdef HAVE_LIBEXIF
+	const char        *gconf_key;
+	GthCaptionFields   caption = 0;
+#else
+	GtkWidget         *vbox;
+#endif
+	cdata = g_new (CaptionDialogData, 1);
+
+	cdata->data = tdata->data;
+	cdata->window = tdata->window;
+	cdata->thumbnail_caption = thumbnail_caption;
+
+	cdata->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_EXPORTER_FILE, NULL, NULL);
+        if (!cdata->gui) {
+		g_free (cdata);
+                g_warning ("Could not find " GLADE_EXPORTER_FILE "\n");
+                return;
+        }
+
+	/* Get the widgets. */
+
+	cdata->dialog = glade_xml_get_widget (cdata->gui, "caption_dialog");
+	cdata->c_comment_checkbutton = glade_xml_get_widget (cdata->gui, "c_comment_checkbutton");
+	cdata->c_imagedim_checkbutton = glade_xml_get_widget (cdata->gui, "c_imagedim_checkbutton");
+	cdata->c_filename_checkbutton = glade_xml_get_widget (cdata->gui, "c_filename_checkbutton");
+	cdata->c_filesize_checkbutton = glade_xml_get_widget (cdata->gui, "c_filesize_checkbutton");
+	cdata->c_exif_date_time_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_date_time_checkbutton");
+	cdata->c_exif_exposure_time_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_exposure_time_checkbutton");
+	cdata->c_exif_exposure_mode_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_exposure_mode_checkbutton");
+	cdata->c_exif_flash_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_flash_checkbutton");
+	cdata->c_exif_shutter_speed_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_shutter_speed_checkbutton");
+	cdata->c_exif_aperture_value_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_aperture_value_checkbutton");
+	cdata->c_exif_focal_length_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_focal_length_checkbutton");
+	cdata->c_exif_camera_model_checkbutton = glade_xml_get_widget (cdata->gui, "c_exif_camera_model_checkbutton");
+
+	ok_button = glade_xml_get_widget (cdata->gui, "c_okbutton");
+	cancel_button = glade_xml_get_widget (cdata->gui, "c_cancelbutton");
+
+	/* Signals. */
+
+	g_signal_connect (G_OBJECT (cdata->dialog), 
+			  "destroy",
+			  G_CALLBACK (caption_dialog_destroy_cb),
+			  cdata);
+	g_signal_connect_swapped (G_OBJECT (cancel_button), 
+				  "clicked",
+				  G_CALLBACK (gtk_widget_destroy),
+				  G_OBJECT (cdata->dialog));
+	g_signal_connect (G_OBJECT (ok_button), 
+			  "clicked",
+			  G_CALLBACK (caption_dialog__ok_clicked),
+			  cdata);
+
+	/* Set widgets data. */
+
+#ifdef HAVE_LIBEXIF
+
+	if (cdata->thumbnail_caption) 
+		gconf_key = PREF_WEB_ALBUM_INDEX_CAPTION;
+	 else 
+		gconf_key = PREF_WEB_ALBUM_IMAGE_CAPTION;
+	caption = eel_gconf_get_integer (gconf_key);
+
+	if (caption & GTH_CAPTION_COMMENT)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_IMAGE_DIM)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_FILE_NAME)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_filename_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_FILE_SIZE)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_filesize_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_DATE_TIME)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_date_time_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_EXPOSURE_TIME)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_exposure_time_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_EXPOSURE_MODE)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_exposure_mode_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_FLASH)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_flash_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_SHUTTER_SPEED)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_shutter_speed_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_APERTURE_VALUE)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_aperture_value_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_FOCAL_LENGTH)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_focal_length_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_EXIF_CAMERA_MODEL)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_camera_model_checkbutton), TRUE);
+
+#else /* ! HAVE_LIBEXIF */
+	vbox = glade_xml_get_widget (cdata->gui, "c_exif_data_vbox");
+	gtk_widget_hide (vbox);
+#endif /* ! HAVE_LIBEXIF */
+
+	/* Run dialog. */
+
+	gtk_window_set_transient_for (GTK_WINDOW (cdata->dialog), GTK_WINDOW (tdata->dialog));
+	gtk_window_set_modal (GTK_WINDOW (cdata->dialog), TRUE);
+	gtk_widget_show (cdata->dialog);
+}
+
+
+static void
+show_thumbnail_caption_dialog_cb (GtkWidget       *widget,
+				  ThemeDialogData *data)
+{
+	show_caption_dialog_cb (widget, data, TRUE);
+}
+
+
+static void
+show_image_caption_dialog_cb (GtkWidget       *widget,
+			      ThemeDialogData *data)
+{
+	show_caption_dialog_cb (widget, data, FALSE);
+}
