@@ -301,15 +301,17 @@ initialize_data (poptContext pctx)
 		/* Create a catalog with the command line list. */
 		Catalog *catalog;
 		char    *catalog_path;
-		char    *catalog_name;
+		char    *catalog_name, *catalog_name_utf8;
 
 		catalog = catalog_new ();
 
-		catalog_name = g_strconcat (_("Command Line"),
-					    CATALOG_EXT,
-					    NULL);
+		catalog_name_utf8 = g_strconcat (_("Command Line"),
+						 CATALOG_EXT,
+						 NULL);
+		catalog_name = g_locale_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
 		catalog_path = get_catalog_full_path (catalog_name);
 		g_free (catalog_name);
+		g_free (catalog_name_utf8);
 
 		catalog_set_path (catalog, catalog_path);
 		g_free (catalog_path);
@@ -469,16 +471,11 @@ all_windows_update_browser_options ()
 
 	for (scan = window_list; scan; scan = scan->next) {
 		GThumbWindow  *window = scan->data;
-		GthViewMode    view_mode;
 		
 		window->file_list->enable_thumbs = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS);
 		gth_file_list_set_thumbs_size (window->file_list, eel_gconf_get_integer (PREF_THUMBNAIL_SIZE));
-		if (eel_gconf_get_boolean (PREF_SHOW_COMMENTS))
-			view_mode = GTH_VIEW_MODE_ALL;
-		else
-			view_mode = GTH_VIEW_MODE_LABEL;
 		gth_file_view_set_view_mode (window->file_list->view,
-					     view_mode);
+					     pref_get_view_mode ());
 		window_update_file_list (window);
 		dir_list_update_underline (window->dir_list);
 	}
